import {
    logError,
    logInfo,
    readConfig,
} from './utils.js';

import {
    db,
    setUserState,
    selectProfilesForUser,
    setUserCurrentlyViewingProfile,
    getProfileInfo,
    getAllLikesForUser,
    checkForMutualLike,
    markLikeAsRead,
    getUserLanguage
} from "./db.js";

import fs from 'fs';


import { I18n } from "i18n-js";
const i18n = new I18n({
    en: JSON.parse(fs.readFileSync("./translations/en.json")),
    ru: JSON.parse(fs.readFileSync("./translations/ru.json"))
});

i18n.defaultLocale = "en";

const requiresLengths = {
    "country": 64,
    "city": 64,
    "description": 512,
    "name": 32,
};

const processRequest = async (client, roomId, question, answer, nextQuestion) => {

    let preferredLanguage = await getUserLanguage(roomId);
    if (!preferredLanguage) preferredLanguage = i18n.defaultLocale;
    i18n.locale = preferredLanguage;

    if (answer.length > requiresLengths[question]) {
        await client.sendText(roomId, i18n.t(["errors", "toobig"]));
        return;
    }
    await db.query(`UPDATE users SET ${question} = $1 WHERE room_id = $2`, [answer, roomId]);
    await client.sendText(roomId, i18n.t(["general", "setopt"], { opt: answer }));

    await client.sendText(roomId, i18n.t(["setup", nextQuestion]));

    setUserState(roomId, nextQuestion);
};

const showRandomProfileToUser = async (client, roomId) => {
    let preferredLanguage = await getUserLanguage(roomId);
    if (!preferredLanguage) preferredLanguage = i18n.defaultLocale;
    i18n.locale = preferredLanguage;

    let chosenProfile = await selectProfilesForUser(roomId);
    if (!chosenProfile) {
        await client.sendText(roomId, i18n.t(["errors", "noprofiles"]));

        return;
    }
    let message =
        `${chosenProfile.country}, ${chosenProfile.city}.
${chosenProfile.name}, ${chosenProfile.sex == 'm' ? 'male' : 'female'}, ${chosenProfile.age}.
${chosenProfile.description}`;

    await setUserCurrentlyViewingProfile(roomId, chosenProfile.room_id);
    await client.sendText(roomId, message);

    if (chosenProfile.media) {
        for (let media of chosenProfile.media) {
            let msgtype;
            if (media.type == 'p') {
                msgtype = "m.image";
            } else if (media.type == 'v') {
                msgtype = "m.video";
            }
            await client.sendMessage(roomId, {
                msgtype: msgtype,
                body: "Profile media",
                url: media.url
            });
        }
    }

    await client.sendText(roomId, i18n.t(["general", "rate"]));
};

const showProfileToUser = async (client, roomId, profileId) => {
    let preferredLanguage = await getUserLanguage(roomId);
    if (!preferredLanguage) preferredLanguage = i18n.defaultLocale;
    i18n.locale = preferredLanguage;

    let profileInfo = await getProfileInfo(profileId);
    let message =
        `${profileInfo.country}, ${profileInfo.city}.
${profileInfo.name}, ${profileInfo.sex == 'm' ? 'male' : 'female'}, ${profileInfo.age}.
${profileInfo.description}`;

    await client.sendText(roomId, i18n.t(["general", "showalike"]));

    await client.sendText(roomId, message);

    if (profileInfo.media) {
        for (let media of profileInfo.media) {
            let msgtype;
            if (media.type == 'p') {
                msgtype = "m.image";
            } else if (media.type == 'v') {
                msgtype = "m.video";
            }
            await client.sendMessage(roomId, {
                msgtype: msgtype,
                body: "Profile media",
                url: media.url
            });
        }
    }
    await client.sendText(roomId, i18n.t(["general", "mxid"], { mxid: profileInfo.mx_id }));

};

const showNewLikes = async (client, roomId) => {
    let likes = (await getAllLikesForUser(roomId));
    for (let liked of likes) {
        if (await checkForMutualLike(liked.sender, roomId)) {
            await showProfileToUser(client, roomId, liked.sender);
            await markLikeAsRead(liked.sender, roomId);
        }
    }
};

export { processRequest, showRandomProfileToUser, showNewLikes, showProfileToUser };